@@ -45,6 +45,8 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkitemfactory.h>
+#include <gtk/gtkmenubar.h>
 #include <libgnomeprint/private/gnome-print-private.h>
 
 #include "gnome-print-i18n.h"
@@ -61,6 +63,23 @@
 #define TOOLBAR_BUTTON_BASE 5
 #define MOVE_INDEX 5
 
+/* These are the callback action identifers for the GtkItemFactory used for
+   the menubar. */
+enum {
+	PREVIEW_PRINT_ACTION,
+	PREVIEW_CLOSE_ACTION,
+
+	PREVIEW_GOTO_FIRST_ACTION,
+	PREVIEW_GO_BACK_ACTION,
+	PREVIEW_GO_FORWARD_ACTION,
+	PREVIEW_GOTO_LAST_ACTION,
+
+	PREVIEW_ZOOM_100_ACTION,
+	PREVIEW_ZOOM_FIT_ACTION,
+	PREVIEW_ZOOM_IN_ACTION,
+	PREVIEW_ZOOM_OUT_ACTION
+};
+
 struct _GnomePrintMasterPreview {
 	GtkWindow window;
 	/* Main VBox */
@@ -106,6 +125,15 @@ struct _GPMPPrivate {
 	gint current_page;
 	gint pagecount;
 
+	GtkWidget *goto_first_menuitem;
+	GtkWidget *go_back_menuitem;
+	GtkWidget *go_forward_menuitem;
+	GtkWidget *goto_last_menuitem;
+
+	GtkWidget *zoom_100_menuitem;
+	GtkWidget *zoom_fit_menuitem;
+	GtkWidget *zoom_in_menuitem;
+	GtkWidget *zoom_out_menuitem;
 };
 
 static void gpmp_parse_layout (GnomePrintMasterPreview *mp);
@@ -159,16 +187,27 @@ goto_page (GnomePrintMasterPreview *mp, gint page)
 {
 	GPMPPrivate *priv;
 	guchar c[32];
+	gboolean can_go_back = FALSE, can_go_forward = FALSE;
 
 	priv = (GPMPPrivate *) mp->priv;
 
 	g_snprintf (c, 32, "%d", page + 1);
 	gtk_entry_set_text (GTK_ENTRY (priv->page_entry), c);
 
-	gtk_widget_set_sensitive (mp->bpf, (page != 0) && priv->pagecount > 1);
-	gtk_widget_set_sensitive (mp->bpp, (page != 0) && priv->pagecount > 1);
-	gtk_widget_set_sensitive (mp->bpn, (page != (mp->pages - 1)) && priv->pagecount > 1);
-	gtk_widget_set_sensitive (mp->bpl, (page != (mp->pages - 1)) && priv->pagecount > 1);
+	if ((page != 0) && priv->pagecount > 1)
+		can_go_back = TRUE;
+	if ((page != (mp->pages - 1)) && priv->pagecount > 1)
+		can_go_forward = TRUE;
+
+	gtk_widget_set_sensitive (mp->bpf, can_go_back);
+	gtk_widget_set_sensitive (mp->bpp, can_go_back);
+	gtk_widget_set_sensitive (priv->goto_first_menuitem, can_go_back);
+	gtk_widget_set_sensitive (priv->go_back_menuitem, can_go_back);
+
+	gtk_widget_set_sensitive (mp->bpn, can_go_forward);
+	gtk_widget_set_sensitive (mp->bpl, can_go_forward);
+	gtk_widget_set_sensitive (priv->go_forward_menuitem, can_go_forward);
+	gtk_widget_set_sensitive (priv->goto_last_menuitem, can_go_forward);
 
 	if (page != priv->current_page) {
 		priv->current_page = page;
@@ -201,6 +240,7 @@ gpmp_zoom (GnomePrintMasterPreview *mp, gdouble factor, gboolean relative)
 {
 	GPMPPrivate *priv;
 	gdouble zoom;
+	gboolean can_zoom_100 = TRUE, can_zoom_in = TRUE, can_zoom_out = TRUE;
 
 	priv = (GPMPPrivate *) mp->priv;
 
@@ -212,9 +252,20 @@ gpmp_zoom (GnomePrintMasterPreview *mp, gdouble factor, gboolean relative)
 
 	mp->zoom = CLAMP (zoom, GPMP_ZOOM_MIN, GPMP_ZOOM_MAX);
 
-	gtk_widget_set_sensitive (mp->bz1, (!CLOSE_ENOUGH (mp->zoom, 1.0)));
-	gtk_widget_set_sensitive (mp->bzi, (!CLOSE_ENOUGH (mp->zoom, GPMP_ZOOM_MAX)));
-	gtk_widget_set_sensitive (mp->bzo, (!CLOSE_ENOUGH (mp->zoom, GPMP_ZOOM_MIN)));
+	if (CLOSE_ENOUGH (mp->zoom, 1.0))
+		can_zoom_100 = FALSE;
+	if (CLOSE_ENOUGH (mp->zoom, GPMP_ZOOM_MAX))
+		can_zoom_in = FALSE;
+	if (CLOSE_ENOUGH (mp->zoom, GPMP_ZOOM_MIN))
+		can_zoom_out = FALSE;
+
+	gtk_widget_set_sensitive (mp->bz1, can_zoom_100);
+	gtk_widget_set_sensitive (mp->bzi, can_zoom_in);
+	gtk_widget_set_sensitive (mp->bzo, can_zoom_out);
+
+	gtk_widget_set_sensitive (priv->zoom_100_menuitem, can_zoom_100);
+	gtk_widget_set_sensitive (priv->zoom_in_menuitem, can_zoom_in);
+	gtk_widget_set_sensitive (priv->zoom_out_menuitem, can_zoom_out);
 
 	gnome_canvas_set_pixels_per_unit (priv->canvas, mp->zoom);
 }
@@ -528,17 +579,14 @@ static void
 create_preview_canvas (GnomePrintMasterPreview *mp)
 {
 	GPMPPrivate *priv;
-	GtkWidget *status, *l;
 	GnomeCanvasItem *item;
 	GnomePrintConfig *config;
-	AtkRelationSet *relation_set;
-	AtkRelation *relation;
-	AtkObject *relation_targets[1];
 	AtkObject *atko;
 
 	priv = (GPMPPrivate *) mp->priv;
 
 	priv->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (priv->scrolled_window), GTK_SHADOW_IN);
 
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 	priv->canvas = GNOME_CANVAS (gnome_canvas_new_aa ());
@@ -581,6 +629,25 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 	gnome_canvas_item_lower_to_bottom (item);
 	gnome_canvas_set_scroll_region (priv->canvas, 0 - PAGE_PAD, 0 - PAGE_PAD, mp->paw + PAGE_PAD, mp->pah + PAGE_PAD);
 
+	gtk_box_pack_start (GTK_BOX (mp->vbox), priv->scrolled_window, TRUE, TRUE, 0);
+
+	gtk_widget_show_all (mp->vbox);
+
+	gtk_widget_grab_focus (GTK_WIDGET (priv->canvas));
+}
+
+static GtkWidget*
+create_page_number_field (GnomePrintMasterPreview *mp)
+{
+	GPMPPrivate *priv;
+	GtkWidget *status, *l, *pad;
+	AtkObject *atko;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+
+	priv = (GPMPPrivate *) mp->priv;
+
 	status = gtk_hbox_new (FALSE, 0);
 	l = gtk_label_new_with_mnemonic (_ ("P_age: "));
 	gtk_box_pack_start (GTK_BOX (status), l, FALSE, FALSE, 4);
@@ -600,8 +667,8 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 	atk_object_set_name (atko, _("Page total"));
 	atk_object_set_description (atko, _("The total number of pages in the document"));
 
-	gtk_box_pack_start (GTK_BOX (mp->vbox), status, FALSE, FALSE, 3);
-	gtk_box_pack_start (GTK_BOX (mp->vbox), priv->scrolled_window, TRUE, TRUE, 0);
+	pad = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (status), pad, FALSE, FALSE, 4);
 
 	/* Add a LABELLED_BY relation from the page entry to the label. */
 	atko = gtk_widget_get_accessible (priv->page_entry);
@@ -613,9 +680,9 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 	g_object_unref (G_OBJECT (relation));
 	g_object_unref (G_OBJECT (relation_set));
 
-	gtk_widget_show_all (mp->vbox);
+	gtk_widget_show_all (status);
 
-	gtk_widget_grab_focus (GTK_WIDGET (priv->canvas));
+	return status;
 }
 
 #ifdef GPMP_VERBOSE
@@ -628,12 +695,101 @@ create_preview_canvas (GnomePrintMasterPreview *mp)
 #define PRINT_AFFINE(s,a)
 #endif
 
+/* This is the GtkItemFactory callback used for all the menubar items.
+   We just need to call the existing toolbar button callbacks and rearrange
+   the arguments a little. */
+static void
+item_factory_cb (GnomePrintMasterPreview *pmp, guint callback_action,
+		 GtkWidget *widget)
+{
+	switch (callback_action) {
+	case PREVIEW_PRINT_ACTION:
+		preview_file_print_cmd (NULL, pmp);
+		break;
+	case PREVIEW_CLOSE_ACTION:
+		preview_close_cmd (NULL, pmp);
+		break;
+	case PREVIEW_GOTO_FIRST_ACTION:
+		preview_first_page_cmd (NULL, pmp);
+		break;
+	case PREVIEW_GO_BACK_ACTION:
+		preview_prev_page_cmd (NULL, pmp);
+		break;
+	case PREVIEW_GO_FORWARD_ACTION:
+		preview_next_page_cmd (NULL, pmp);
+		break;
+	case PREVIEW_GOTO_LAST_ACTION:
+		preview_last_page_cmd (NULL, pmp);
+		break;
+	case PREVIEW_ZOOM_100_ACTION:
+		preview_zoom_100_cmd (NULL, pmp);
+		break;
+	case PREVIEW_ZOOM_FIT_ACTION:
+		preview_zoom_fit_cmd (NULL, pmp);
+		break;
+	case PREVIEW_ZOOM_IN_ACTION:
+		gpmp_zoom_in_cmd (NULL, pmp);
+		break;
+	case PREVIEW_ZOOM_OUT_ACTION:
+		gpmp_zoom_out_cmd (NULL, pmp);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static GtkItemFactoryEntry menu_items[] =
+{
+  { N_("/_Preview"),			NULL,
+    0,					0, "<Branch>" },
+  { N_("/Preview/_Print"),		NULL,
+    item_factory_cb,			PREVIEW_PRINT_ACTION,
+    "<StockItem>",			GTK_STOCK_PRINT },
+  { N_("/Preview/_Close"),		NULL,
+    item_factory_cb,			PREVIEW_CLOSE_ACTION,
+    "<StockItem>",			GTK_STOCK_CLOSE },
+
+  { N_("/_View"),			NULL,
+    0,					0, "<Branch>" },
+  { N_("/View/Zoom _1:1"),		"<Ctrl>1",
+    item_factory_cb,			PREVIEW_ZOOM_100_ACTION,
+    "<StockItem>",			GTK_STOCK_ZOOM_100 },
+  { N_("/View/Zoom to _Fit"),		"<Ctrl>F",
+    item_factory_cb,			PREVIEW_ZOOM_FIT_ACTION,
+    "<StockItem>",			GTK_STOCK_ZOOM_FIT },
+  { N_("/View/Zoom _In"),		"<Ctrl>plus",
+    item_factory_cb,			PREVIEW_ZOOM_IN_ACTION,
+    "<StockItem>",			GTK_STOCK_ZOOM_IN },
+  { N_("/View/Zoom _Out"),		"<Ctrl>minus",
+    item_factory_cb,			PREVIEW_ZOOM_OUT_ACTION,
+    "<StockItem>",			GTK_STOCK_ZOOM_OUT },
+
+  { N_("/_Go"),				NULL,
+    0,					0, "<Branch>" },
+  { N_("/Go/_First Page"),		"Home",
+    item_factory_cb,			PREVIEW_GOTO_FIRST_ACTION,
+    "<StockItem>",			GTK_STOCK_GOTO_FIRST },
+  { N_("/Go/_Previous Page"),		"<Alt>Left",
+    item_factory_cb,			PREVIEW_GO_BACK_ACTION,
+    "<StockItem>",			GTK_STOCK_GO_BACK },
+  { N_("/Go/_Next Page"),		"<Alt>Right",
+    item_factory_cb,			PREVIEW_GO_FORWARD_ACTION,
+    "<StockItem>",			GTK_STOCK_GO_FORWARD },
+  { N_("/Go/_Last Page"),		"End",
+    item_factory_cb,			PREVIEW_GOTO_LAST_ACTION,
+    "<StockItem>",			GTK_STOCK_GOTO_LAST },
+};
+
 static void
 create_toplevel (GnomePrintMasterPreview *mp)
 {
 	GPMPPrivate *priv;
 	GtkWidget *tb;
 	gint width, height;
+	GtkItemFactory *item_factory;
+	GtkWidget *menubar;
+	GtkAccelGroup *accel_group;
+	GtkWidget *page_number_field;
 
 	priv = (GPMPPrivate *) mp->priv;
 
@@ -643,6 +799,39 @@ create_toplevel (GnomePrintMasterPreview *mp)
 	gtk_window_set_policy (GTK_WINDOW (mp), TRUE, TRUE, FALSE);
 	gtk_widget_set_usize (GTK_WIDGET (mp), width, height);
 
+	/* Create the menubar items, using the factory. */
+	accel_group = gtk_accel_group_new ();
+	gtk_window_add_accel_group (GTK_WINDOW (mp), accel_group);
+	g_object_unref (accel_group);
+	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
+					     accel_group);
+	gtk_item_factory_set_translate_func (item_factory, 
+					     (GtkTranslateFunc) libgnomeprintui_gettext,
+					     NULL, NULL);
+	gtk_item_factory_create_items (item_factory, G_N_ELEMENTS (menu_items),
+				       menu_items, mp);
+	menubar = gtk_item_factory_get_widget (item_factory, "<main>");
+	gtk_widget_show (menubar);
+	gtk_box_pack_start (GTK_BOX (mp->vbox), menubar, FALSE, FALSE, 0);
+
+	priv->goto_first_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_GOTO_FIRST_ACTION);
+	priv->go_back_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_GO_BACK_ACTION);
+	priv->go_forward_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_GO_FORWARD_ACTION);
+	priv->goto_last_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_GOTO_LAST_ACTION);
+
+	priv->zoom_100_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_ZOOM_100_ACTION);
+	priv->zoom_fit_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_ZOOM_FIT_ACTION);
+	priv->zoom_in_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_ZOOM_IN_ACTION);
+	priv->zoom_out_menuitem = gtk_item_factory_get_item_by_action
+		(item_factory, PREVIEW_ZOOM_OUT_ACTION);
+
 	/* This is not very beautiful, but works for now */
 
 	tb = gtk_toolbar_new ();
@@ -660,6 +849,12 @@ create_toplevel (GnomePrintMasterPreview *mp)
 
 	gtk_toolbar_append_space (GTK_TOOLBAR (tb));
 
+	page_number_field = create_page_number_field (mp);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (tb), page_number_field,
+				   NULL, NULL);
+
+	gtk_toolbar_append_space (GTK_TOOLBAR (tb));
+
 	mp->bpf = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GOTO_FIRST, _("Shows the first page"), "", 
 					    GTK_SIGNAL_FUNC (preview_first_page_cmd), mp, -1);
 	mp->bpp = gtk_toolbar_insert_stock (GTK_TOOLBAR (tb), GTK_STOCK_GO_BACK, _("Shows the previous page"), "", 
@@ -810,12 +1005,9 @@ gnome_print_master_preview_new (GnomePrintMaster *gpm, const guchar *title)
 	
 	if (priv->pagecount == 0) {
 		priv->pagecount = 1;
-		text = g_strdup (_("<markup>1   <span foreground=\"red\" "
-				   "weight=\"ultrabold\" background=\"white\">"
-				   "No visible output was created.</span></markup>"));
-	} else 
-		text =  g_strdup_printf ("%d", priv->pagecount);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (priv->last), text);
+	}
+	text = g_strdup_printf ("%d", priv->pagecount);
+	gtk_label_set_text (GTK_LABEL (priv->last), text);
 	g_free (text);
 
 	return (GtkWidget *) gpmp;
