@@ -8,7 +8,7 @@ G_BEGIN_DECLS
 typedef struct _GnomePrintPreviewPrivate GnomePrintPreviewPrivate;
 
 #include <libgnomecanvas/libgnomecanvas.h>
-#include <libgnomeprint/gnome-print-private.h>
+#include <libgnomeprint/private/gnome-print-private.h>
 #include "gnome-print-preview.h"
 
 struct _GnomePrintPreview {
