@@ -1,15 +1,6 @@
-#define __GNOME_PRINTER_DIALOG_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 1999-2001 Ximian Inc. and authors
- *
- *  Authors:
- *    Raph Levien (raph@acm.org)
- *    Miguel de Icaza (miguel@kernel.org)
- *    Lauris Kaplinski <lauris@ximian.com>
- *    Chema Celorio (chema@celorio.com)
- *
- *  Printer selection widget and dialog
+ *  gnome-printer-dialog.c:
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -24,8 +15,19 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Raph Levien (raph@acm.org)
+ *    Miguel de Icaza (miguel@kernel.org)
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *    Chema Celorio (chema@celorio.com)
+ *
+ *  Copyright (C) 1999-2002 Ximian Inc. and authors
+ *
  */
 
+#define __GNOME_PRINTER_DIALOG_C__
+
 #include <config.h>
 
 #include <atk/atkobject.h>
@@ -178,7 +180,7 @@ gnome_printer_selection_init (GtkObject *object)
 	   and the entry. */
 	label_atko = gtk_widget_get_accessible (l);
 	menu_atko = gtk_widget_get_accessible (((GPATransportSelector*) gpw->transport)->menu);
-	entry_atko = gtk_widget_get_accessible (((GPATransportSelector*) gpw->transport)->entry);
+	entry_atko = gtk_widget_get_accessible (((GPATransportSelector*) gpw->transport)->fentry);
 
 	relation_set = atk_object_ref_relation_set (label_atko);
 	relation_targets[0] = menu_atko;
@@ -253,18 +255,6 @@ gnome_printer_selection_new (GnomePrintConfig *config)
 	return GTK_WIDGET (gpw);
 }
 
-#if 0
-void
-gnome_printer_selection_bind_editable_enters (GnomePrinterSelection * gpw, GnomeDialog * dialog)
-{
-}
-
-void
-gnome_printer_selection_bind_accel_group (GnomePrinterSelection * gpw, GtkWindow * window)
-{
-}
-#endif
-
 GnomePrintConfig *
 gnome_printer_selection_get_config (GnomePrinterSelection *widget)
 {
@@ -470,63 +460,11 @@ gnome_printer_dialog_new (GnomePrintConfig *config)
 			    GTK_WIDGET (pd->gnome_printer_selection), TRUE, TRUE, 0);
 	gpw = pd->gnome_printer_selection;
 
-#ifdef	ENABLE_LIBGPA
-#else	
-#if 0
-	gnome_dialog_editable_enters (GNOME_DIALOG(pd),
-				      GTK_EDITABLE(gpw->entry_command));
-	gnome_dialog_editable_enters (GNOME_DIALOG(pd),
-				      GTK_EDITABLE(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (gpw->entry_filename))));
-#endif
-#endif
-#if 0	
-	gtk_widget_grab_focus (gpw->profile_selector);
-#endif
 	gtk_window_add_accel_group (GTK_WINDOW (pd), gpw->accel_group);
 	
 	return printer_dialog;
 }
 
-#if 0
-/**
- * gnome_printer_dialog_new_modal:
- *
- * Runs a dialog that allows the user to select a target
- * printer and returns a GnomePrinter object.  It returns NULL
- * on failure or user cancel.
- *
- * This function runs the dialog and returns the user selection.
- *
- * Returns: A GnomePrinter object that represents the printer
- * selected by the user or NULL if the user cancels the operation.
- */
-GnomePrinter *
-gnome_printer_dialog_new_modal (void)
-{
-	GtkWidget *printer_dialog;
-	GnomePrinterDialog *pd;
-	GnomePrinter *printer;
-	int button;
-
-	printer_dialog = gnome_printer_dialog_new();
-	pd = GNOME_PRINTER_DIALOG (printer_dialog);
-	
-	gtk_window_set_modal (GTK_WINDOW (printer_dialog), TRUE);
-	button = gnome_dialog_run (GNOME_DIALOG (printer_dialog));
-	
-	if (button < 0)
-		return NULL;
-	
-	printer = NULL;
-	if (button == 0)
-		printer = gnome_printer_dialog_get_printer (pd);
-	
-	gtk_widget_destroy (printer_dialog);
-
-	return printer;
-}
-#endif
-
 /**
  * gnome_printer_dialog_get_printer:
  * @dialog: a GnomePrinterDialog
